#! /usr/bin/env python

############################## ALPHA2 Version ################################

"""" Email Manager to sweep unwanted emails from the servers or back them up
Usage = program [configfile] (optional after first time or when changing config)
The config file allows some flexibility to assign name for the server's section
names and for the filter names
Server sections have free choice of name and have following items:
	SERVER =
	USER =
	PASS =
	PROTOCOL =
Filters section is fixed name and free option name. Just they must start
with A or D which means A'llow the emails to keep, D'eny those to remove
	A01 = regex pattern
	A02 = more regex
	D01 = regex of denial
	D02 = more denial

	Setting section has a fix name and take fixed option names

	TODO:
	o	Rotating file log or a syslog
	o	download wanted msg for backup and removing the rest or all or just
		keep the good messages on the server and clean the others
	o	Way of gathering local emails and save them for later use, maybe a
		local IMAP4 proxy will improve the handling
	o	GUI and some visual effect """

__author__ = "Fulvio Cogrossi"

import poplib, imaplib, socket
from email.Parser import HeaderParser
import sys , os , re, time
import os.path as Path
import utilities, mhandler
from optparse import OptionParser
from cfgparse import ConfigParser as CParser

try:
	import cPickle
except:
	import Pickle as cPickle

# Program Variables

# find working directory and root file name, without version dotted style
pgmhmdir, pgmname = Path.split(Path.realpath(sys.argv[0]))
progname = re.sub(r'(?<!\.)\.\d*.*$','',pgmname)
EOL = os.linesep
SLH = Path.sep
Exists = utilities.Exists
myhome= utilities.get_homedir()
chksum= utilities.chksum
_pickdata= utilities.pickdata
_loadcfg= utilities._ldcfg
_parseCli= utilities.parseCli
dsplog= utilities.log().logger
IS_WIN = (sys.platform == 'win32')
PklData= 'stordata.dat'
class ProtocolError(Exception):
	pass

def _setter():
	dbg= cfgfil['setting.log']; mode= 'w' # normally write, it writes new
	try:
		if cfgfil['setting.append'] > Path.getsize(dbg): mode= 'a'
	except (OSError, TypeError): # found a boolean or dbg is new file
		pass
	er=  cfgfil['setting.filter']
	# flexibility cost some effort to find the all setting into its section
	al= re.compile('|'.join([k[1] for k in cfgfil['filters'] if k[0].startswith('a')]))
	dn= re.compile('|'.join([k[1] for k in cfgfil['filters'] if k[0].startswith('d')]))
	st= {}
	for adir, sect in enumerate([k for k in cfgfil if k.startswith('server.')]):
		st[adir]= cfgfil[sect]
	st[adir+1]= (dbg, mode)
	st[adir+2]= (al, dn, er)
	return adir+ 1, st

def snctr(syncmbx):
	# Suppose the path has spaces within it's allowed to close into quote chars (')
	if syncmbx.startswith("'"): syncmbx= syncmbx.strip("'")
	if Exists(syncmbx):
		import mbxreader as mb
		return mb.get_trash(syncmbx)
	return None

# Mailsweeper starts to guess from where gather the configuration setting.
# If it isn't from the CLI will try to find in the same directory of the
# program or trying to load a pickled configfile
version = sys.version_info[:2]
if version < (2, 3):
	print 'Sorry, need Python 2.3 or better; %s.%s is too old!' % version
	sys.exit()
chksm= None; numargv= len(sys.argv)
try: # load the configfile from command line if there is one
	inifil= sys.argv[1]
	cfgfil= _loadcfg(inifil)
	chksm= chksum(inifil)
	cfgfil['setting.config'] = inifil
except (IOError, IndexError):
	try: # load from pickle file
		stdata= open(pgmhmdir + SLH +PklData, 'rb')
		cfgfil= cPickle.load(stdata)
		stdata.close()
		chksm= chksum(cfgfil['setting.config'])
	except (KeyError, IOError, cPickle.UnpicklingError):
		try: # load from current directory
			lclini= ('%s%s%s.ini' %(pgmhmdir, SLH, progname))
			cfgfil= _loadcfg(lclini)
			chksm= chksum(lclini)
			cfgfil['setting.workdir']= Path.split(os.getcwd())[1]
		except (IOError, IndexError, TypeError):
			print 'Unable to proceed, missing needed data'
			sys.exit(1)
# Trying to guess if files have been changed since last time
# if nothing has changed it will prefer the pickled setting

cksum= cfgfil['setting.checksum']
if cksum != chksm:
	lastpos, setting = _setter()
	cfgfil['setting.last'] = lastpos
	# this is written only on the variable cfgfil, never on the file
	cfgfil['setting.checksum'] = chksm
else:
	# Nothing changed it load from pickle file. If it had the cfgfil from the
	# pickle file it can fetch the rest as before
	pth= pgmhmdir + SLH + PklData
	if cfgfil['setting.savedata']: pth= cfgfil['setting.savedata']
	try:
		cfgfil, setting= _pickdata(pth)
		cfgfil['setting.savedata']= pth # goes only on pickle file
	except IOError:
		print 'Error loading data'
		while 1: # it gives a chance to ask for a pickle data file
			o= raw_input('Need a saved data file to load configuration or quit ')
			if Exists(o): cfgfil, setting= _pickdata(o); break
			elif 'quit' in o.lower(): sys.exit(2)
			else:
				print 'Unclear command or path.Type "quit" or a good file path\n'
# If CLI has options, than it will change the configuration

if numargv > 1:
	for cnt in range(numargv):
		if sys.argv[cnt].startswith('-'):
			_parseCli(cfgfil)
			setting= 0 # ensure to read setting again
			break
synm= cfgfil['setting.trash']; lmbox= None; lastpos= cfgfil['setting.last']
if IS_WIN and 'search' in synm.lower():
	synm = utilities.Walk(myhome, 1, '*deleted items.dbx')
	if len(synm) == 0: synm = None
	else: synm = synm[0]
if synm: lmbox= snctr(synm)
if not setting or not lastpos:
	lastpos, setting = _setter()
	cfgfil['setting.last'] = lastpos
accs= cfgfil['setting.verbose'] ; rx = []
dbg, mode= setting[lastpos][0],setting[lastpos][1]
_logger= utilities.log(dbg,mode).logger
_log_in= mhandler.ConnectHandler(lmbox, _logger, accs)
multhr= cfgfil['setting.multithread']
if multhr:
	_log_in= mhandler.mThreadSession(lmbox, _logger, accs)
for s in range(lastpos):
	if len(setting[s]) < 4: continue # if no protocol, then skip
	nmsvr, user, pwd, ptcl= setting[s][0], setting[s][1], setting[s][2], setting[s][3]
	try: rx.append( _log_in.connect([nmsvr, user, pwd, ptcl, setting[len(setting)-1]]))
	except ProtocolError:
		print nmsvr + '\n\nhad problem during connection\n'
	except KeyboardInterrupt:
		print '\n\nStopped by you\n'
		sys.exit()
if multhr:
	for s in rx:
		try: s.start()
		except ProtocolError:
			print nmsvr + '\n\nhad problem during connection\n'
		except KeyboardInterrupt:
			print '\n\nStopped by you\n'
			sys.exit()
	for s in rx:
		try: s.join() # waiting all threads to finish
		except ProtocolError:
			print nmsvr + '\n\nhad problem during connection\n'
		except KeyboardInterrupt:
			print '\n\nStopped by you\n'
			sys.exit()
pth= cfgfil['setting.savedata']
if not Exists(pth):
	pth= myhome  + SLH + PklData
if IS_WIN:
	if not re.search(r'\\', cfgfil['setting.savedata']):
		pth= pgmhmdir + SLH + PklData
try: _pickdata( pth, (cfgfil, setting))
except IOError: print 'Unable to save pickled data'
dsplog(EOL,accs)
if dbg: _logger('?close?',True)
if IS_WIN:
	answer= raw_input('Close Y/N ')
	if answer.lower() == 'y': os.system('exit')
	else: sys.exit()
