#! /usr/bin/env python

import poplib, imaplib, socket, re, sys, threading,utilities, time, os
from email.Parser import HeaderParser

class ProtocolError(Exception):
	pass

_FULLhdr = ('from', 'to', 'cc', 'bcc', 'date', 'subject', 'reply-to', 'message-id')
_FULLhdrCap = ' '.join(_FULLhdr).upper()
EOL= os.linesep
dsplog= utilities.log().logger

class ConnectHandler:
	''' Class to handle imap or pop3 connections'''
	def __init__(self, lmbox, logger, access):
		self.lmbox= lmbox
		self._log= logger
		self.accs= access

	def connect(self, args):
		if self.argcheck(args):
			print 'Incorrect arguments number'
			sys.exit()
		if 'imap4' in args:
			do_it= 'imap'
		if 'pop3' in args:
			do_it= 'pop3'
		dsplog('Connecting%s' %EOL, self.accs)
		self._log( '%s[%s on %s]%s' %(EOL, args[0], time.ctime(),EOL))
		del args[3]
		eval('self.call_%s(args)'%do_it)
		dsplog(EOL,self.accs)
		return

	def argcheck(self, args):
		nargs= len(args)
		if nargs != 5: return True
		for cnt in range(nargs):
			if args[cnt] =='' or args[cnt] == None : return True
		return False

	def _clnhdr(self, msg, itms= _FULLhdr, mode= 'get'):
		hdrs= []
		if isinstance(itms,str): itms= [itms]
		for itm in itms:
			msgs= eval('msg.%s(itm)' %mode)
			if msgs: # empty items are skipped
				hdr= '%s: %s' %(itm.capitalize(), msgs)
				hdrs.append(hdr)
		return hdrs

	def _filter(self, msg, al= None, dn= None):
		""" Filter try to classify a list of items for a compiled patterns sets"""
		a= 0
		if isinstance(msg,str): msg= [msg]
		for hdrline in msg:
			# deny has the highest priority which won't be overriden by allow
			if dn and dn.search(hdrline):a-= len(msg) * -11
			if al and al.search(hdrline): return 0
			a += 1
		return a

	def _deleter(self, func, argm, numMsg, server, filters, hdrs):
		flag= ''; al, dn, er= filters
		indx= hdrs
		if isinstance(hdrs,list):
			indx= '\r\n'.join(hdrs) # converted in string to find the id
		try:
			indx= re.search('message-id:\s<.*?>',indx, re.I|re.M).group(0)
		except AttributeError:
			indx[indx.find('essage-id: '):]
		indx= indx.split(': ')[1].strip('<>')
		try: # if msg has no ID ignore it
			if self.lmbox and self.lmbox.pop(indx, False):
				# found, no more search on it
				flag= 'DELETed'
		except (ValueError, IndexError): pass
		try:
			# Either if fltr[2]  is a value then the filter erase the message or will
			# only flag it BAD in the report
			if not flag:
				if filters and self._filter(hdrs, al, dn): flag= 'DELETED'
			if flag and not er: flag= 'BAD'
			if flag.startswith('DELET'): func(*argm)
			dsplog('\r%sMsg%04d %s' %(server.ljust(25), numMsg,flag.ljust(9)), \
							self.accs)
		except KeyboardInterrupt:
			raise KeyboardInterrupt
		return  'Msg%04d %s= %s%s' %(numMsg, flag, hdrs, EOL)

	def call_imap(self, args):
		"""Gathering messages from an Imap server and return a report or
		a direct deletion"""
		_notouch= 'rchiv'
		size= re.compile("\d+ \(RFC822.SIZE (\d+)\)")
		total= 0; _imap= utilities.IMAPSession
		try:
			session= _imap(args[0], imaplib.IMAP4_PORT, [_notouch,], True)
		except ProtocolError: return
		session.login(args[1], args[2])
		for imapfld in session.folder_list():
			if _notouch.lower() in imapfld: continue
			session.SelectFolder(imapfld)
			maxlength= cnt= 0
			t, data= session.search(None, 'ALL'); self.itsOK(session,t)
			data= data[0].split()
			if not data: continue
			for cnt in data:
				t, data= session.fetch(cnt,'(BODY[HEADER.FIELDS ('+ _FULLhdrCap+ ')])')
				self.itsOK(session,t); hdrs= data[0][1].strip()
				t, bytes= session.fetch(cnt, '(RFC822.SIZE)'); self.itsOK(session,t)
				length= int(size.match(bytes[0]).group(1))
				maxlength= max(length, maxlength)
				total+= length; cnt= int(cnt)
				b= self._deleter(
				session.store, (cnt, '+FLAGS.SILENT', '(\Deleted)'),
				 cnt, args[0], args[3], hdrs)
				self._log(b)
		self._log('%s#STAT= Server holds %d Messages and %d kbytes Full%s' \
				%(EOL, cnt, int(total/ 1024), EOL))
		session.logout()
		return

	def itsOK(self, session, t): # if the response wasn't OK, we abort from Imap
		if not 'ok' in str(t).lower():
			session.logout()
			raise ProtocolError
		return

	def call_pop3(self, args):
		"""Gathering messages from an pop server and return a report or
		a direct deletion"""
		srvr= args[0]
		try:
			_pop= poplib.POP3(srvr)
			_pop.user(args[1])
			self._log('%s#STAT= POP Server %s reply  %s %s'\
					%(EOL, srvr, _pop.pass_(args[2]), EOL))
		except (poplib.error_proto):
			raise ProtocolError
		except (socket.timeout,socket.gaierror):
			self._log('%s#STAT= POP Server %s failed%s' %(EOL, srvr, EOL))
			try: _pop.quit()
			except NameError: pass # it wasn't started
			return
		(numMsgs, total)= _pop.stat()
		self._log('%s#STAT= Server holds %d Messages and %d Kbytes Full%s'\
					%(EOL, numMsgs, int(total/ 1024), EOL))
		for cnt in range(1, numMsgs +1):
			try: msgs= EOL.join(_pop.top(cnt,0)[1])
			except (poplib.error_proto,socket.timeout,socket.gaierror):
				raise ProtocolError
			#Need a cleanup from the envelope header
			hdrs= self._clnhdr(HeaderParser().parsestr(msgs,True))
			# try to remove messages in sync with local trash and filters
			b= self._deleter(_pop.dele, (cnt,), cnt, srvr, args[3], hdrs)
			self._log(b)
		_pop.quit()
		return

class mThreadSession(ConnectHandler, threading.Thread, utilities.log):
	''' Sub-class to make multithread connection to all servers '''
	vlock = threading.Lock()
	def __init__(self, l, logger, access):
		try :
			a= self.l
		except AttributeError: self.lmbox= l
		self._log= logger
		self.accs= access
		threading.Thread.__init__(self)
# Overriding the normal logger so it can put a lock when writing to
	def logger(self, strng, output=None):
		self.vlock.acquire()
		utilities.log().logger(strng, output)
		self.vlock.release()

	def connect(self, arg):
		if ConnectHandler.argcheck(self, arg):
			print 's% wrong parameters' %arg[0]
			return
# For multithread it should go to this method of logging
		dsplog= self.logger
		if 'imap4' in arg:
			do_it= 'imap'
		if 'pop3' in arg:
			do_it= 'pop3'
		self._log( '%s[%s on %s]%s' %(EOL, arg[0], time.ctime(),EOL))
		del arg[3]
		proc= eval('ConnectHandler.call_' + do_it)
		x = threading.Thread(target=proc, args=(self, tuple(arg)),name=arg[0])
		return x

if __name__ == '__main__':
	pass