#! /usr/bin/env python

import os, sys, md5, signal, re, socket, fnmatch
import os.path as Path
import ConfigParser
from optparse import OptionParser, make_option
from cfgparse import ConfigParser as CParser
from imaplib import IMAP4 as BaseIMAP

class ProtocolError(Exception):
	pass

_boolean_states = {'1': True, 'yes': True, 'true': True, 'on': True,
						'0': False, 'no': False, 'false': False, 'off': False}
try:
	import cPickle
except:
	import Pickle as cPickle

class log:
	""" Output lists and strings on file or any possible device that allows write
	mode. Options are: output file writing mode and special end of line (EOL)"""

	def __init__(self,output= None, mode= None, lsp= None):
		""" Can instantiate even no output is given. Choose to build an open file
		object to append to, or new and give a different line separator"""

		if lsp is None: self.lsp = os.linesep # EOL according to OS platform
		else: self.lsp = lsp
		self.output= output
		self.mode= 'a'
		if mode: self.mode= mode
		if output:
			try:  self.output= open(self.output, self.mode)
			except :self.output = None

	def logger(self, strng, output=None):
		# without any output will simply send log to nowhere :-)
		if not output: return # no allowed to write the output, keep silent
		if isinstance(strng,list):
			a= ''.join(strng); strng= a.replace('\n',self.lsp)
		if isinstance(output,bool) and output is True:
			if strng == '?close?':
				try: self.output.close()
				except IOError: pass # silently ignore the failure
			else:
				sys.stderr.write(strng)
				sys.stderr.flush()
		if isinstance(self.output,file):
			self.output.write(strng)
		return

class IMAPSession(BaseIMAP):
	'''A class extending the IMAP4 class, with a few optimizations'''
	# extrapolated from SpamBayes,sb_imapfilter.py
	# Maximum amount of data that will be read at any one time.
	MAX_SAFE_READ = 4096
	def __init__(self, server, port, no_expunge_fol=[], do_expunge=False):
		try:
			BaseIMAP.__init__(self, server, port)
		except (BaseIMAP.error, socket.gaierror, socket.error):
			print "Cannot connect to server %s on port %s" % (server, port)
			raise ProtocolError
			#sys.exit(-1)
		self.current_folder = None
		self.do_expunge = do_expunge
		self.no_expunge_fol= no_expunge_fol
		self.logged_in = False
		self._read = self.read
		self.read = self.safe_read

	def login(self, username, pwd):
		try:
			BaseIMAP.login(self, username, pwd)  # superclass login
		except BaseIMAP.error, e:
			if str(e) == "permission denied":
				print "There was an error logging in to the IMAP server."
				print "The userid and/or password may be incorrect."
				raise ProtocolError
				#sys.exit()
			else:
				raise
		self.logged_in = True

	def logout(self):
		if self.do_expunge and self.logged_in:
			for fol in self.folder_list():
				if fol.lower() in self.no_expunge_fol: continue
				self.select(fol)
				self.expunge()
		BaseIMAP.logout(self)  # superclass logout

	def SelectFolder(self, folder):
		'''A method to point ensuring imap operations at a target folder'''
		if self.current_folder != folder:
			if self.current_folder != None:
				if self.do_expunge: BaseIMAP.close(self)
			if folder == "":
				print "Tried to select an invalid folder"
				raise ProtocolError
				#sys.exit(-1)
			response = self.select(folder, None)
			if response[0] != "OK":
				print "Invalid response to select %s:\n%s" % (folder, response)
				raise ProtocolError
				#sys.exit(-1)
			self.current_folder = folder
			return response

	def folder_list(self):
		'''Return a alphabetical list of all folders available on the server'''
		r1 = re.compile(r"{\d+}")
		r2 = re.compile(r"\(([\w\\ ]*)\) ")
		response = self.list()
		if response[0] != "OK": return []
		all_folders = response[1]
		folders = []
		for fol in all_folders:
			if isinstance(fol, ()):
				m = r1.search(fol[0])
				if not m: continue
				fol = '%s"%s"' % (fol[0][:m.start()], fol[1])
			m = r2.search(fol)
			if not m: continue
			name_attributes = fol[:m.end()-1]
			self.folder_delimiter = fol[m.end()+1:m.end()+2]
			if self.folder_delimiter == ',':
				print "WARNING: Your imap server uses a comma as the " \
					  "folder delimiter.  This may cause unpredictable " \
					  "errors."
			folders.append(fol[m.end()+4:].strip('"'))
		folders.sort()
		return folders

	def safe_read(self, size):
		"""Read data from remote, but in manageable sizes."""
		data = []
		while size > 0:
			if size < self.MAX_SAFE_READ:
				to_collect = size
			else:
				to_collect = self.MAX_SAFE_READ
			data.append(self._read(to_collect))
			size -= self.MAX_SAFE_READ
		return "".join(data)

def get_homedir():
	if sys.platform == 'win32':
		if os.environ.has_key('USERPROFILE'):
			return os.environ['USERPROFILE']
		else:
			return 'C:\\'
	else:
		if (os.environ.has_key('HOME')):
			return os.environ['HOME']
		else:
			# No home directory set
			return ''

def chksum(fil):
	o= open(fil,'r')
	m= md5.new(); m.update(o.read()); o.close()
	return m.hexdigest()

def Exists(filepath):
	return Path.isfile(filepath) or None

def pickdata(wher,d= None):
	if d != None:
		c, s= d
		try:
			wher= open(wher, 'wb')
			cPickle.dump(c,wher,1)
			cPickle.dump(s,wher,1)
			wher.close()
		except (IOError, cPickle.PicklingError):
			raise IOError
	elif d is None:
	# if no data means to read from the file
		try:
			wher= open(wher, 'rb')
			c= cPickle.load(wher)
			s= cPickle.load(wher)
			wher.close()
		except (IOError, cPickle.UnpicklingError):
			raise IOError
	else: return '',''
	return c, s

def _ldcfg(_file, config={}):
	""" Loads a specific config file into a dictionary """
	if not Exists(_file):
		answer= \
		raw_input('File not found.Continue Y/N,\n if yes will try to find other data  ')
		if answer.lower() == 'n': raise SystemExit
		else:
			raise IOError
	config= config.copy( )
	cp= ConfigParser.ConfigParser()
	cp.read(_file)
	for sec in cp.sections():
		name= sec.lower()
		try:
			srvset = [cp.get(sec, x) for x in ('server', 'user', 'pass', 'protocol')]
			config['server'+ '.' + name] = srvset
		except ConfigParser.NoOptionError: pass
		if name.startswith('filter'):
			config[name] = sorted(cp.items(sec))
		if name.startswith('setting'):
			for opt in cp.options(sec):
				opt= opt.lower()
				config[name + "." + opt]= chkbool(cp.get(sec, opt))
	return parseCli(config)
# This separation let to have a call from the main if the dictionary is already
# set, but found some other setting from CLI.

def parseCli(cfl):
	""" Call to reconfigure the main setting dictionary"""
	opt_lst= [];
	stn_items = 'log', 'append', 'trash', 'download', \
					'filter', 'verbose', 'servers', 'multithread'
	for cnt in cfl:
		if not cnt.endswith('_hlp'): continue
		opt= cnt[8:-4]
		short= 'make_option("-%s" , "--%s" , help="%s" , dest="%s")' \
		%(opt[0], opt, cfl[cnt], opt)
		opt_lst.append(eval(short))
	ps = OptionParser(option_list=opt_lst)
	cp = CParser().parse(ps)[0]
	try: CParser().add_file(cfl['setting.config'])
	except IOError: pass
	for nn in stn_items:
		value= eval('cp.%s' %nn)
		if value: eval('_%s(cfl,value)' %nn)
	return cfl

def chkbool(v):
	if v.lower() not in _boolean_states:
		return v
	return _boolean_states[v.lower()]

def _servers(cnf,op):
	""" Find the server name and cut off the protocol from it,
	so later it won't proceed to perform the poll to its server"""
	strn = []
	for k in op.split(','):
		for j in cnf.keys():
			if j.startswith('serv'):
				q= j.split('.')[1]
				# allowed he first 3 initial letters of the name, too
				if len(k) > 2 and q.startswith(k):
					strn.append('server.' + q)
	for s in list(set(strn)):
		cnf[s]= cnf[s][:-1]
	return cnf

def _log(c,v):
	c['setting.log']= chkbool(v)
	return c

def _append(c,v):
	value= v.lower()
	if value in _boolean_states:
		c['setting.append']= chkbool(v)
		return c
	if re.match(r'^[\d]+(k|m)?$', value):
		k= 1024; M= k * k; v= int(value[:-1])
		if value[-1:] == 'k': v= v * k
		if value[-1:] == 'm': v= v * m
	c['setting.append']= v
	return c

def _trash(c,v):
	c['setting.trash']= chkbool(v)
	return c

def _multithread(c,v):
	c['setting.multithread']= chkbool(v)
	return c

def _filter(c,v):
	if v.lower() in _boolean_states:
		c['setting.filter']= chkbool(v)
		return c
	res= []; flt= ''
	for cnt in v.split():
		if '=' in cnt: flt= tuple(cnt.split('=',1))
		elif ':' in cnt: flt= tuple(cnt.split(':',1))
		res.append(flt)
		c['filters']= res
	return c

def _verbose(c,v):
	c['setting.verbose']= chkbool(v)
	return c

def _download(c,v):
	c['setting.download']= chkbool(v)
	return c

def file_find(folder, fname):
    """search for a filename fname starting in folder"""
    for root, dirs, files in os.walk(folder):
        for file in files:
            # make search case insensitive
            if fname.lower() == file.lower():
                return Path.join(root, fname)

def Walk( root, recurse=0, pattern='*', return_folders=0 ):
	# initialize
	result = []
	# must have at least root folder
	try:
		names = os.listdir(root)
	except os.error:
		return result
	# expand pattern
	pattern = pattern or '*'
	# check each file
	for name in names:
		fullname = Path.normpath(Path.join(root, name))
		# grab if it matches our pattern and entry type
		for pat in pattern.split(';'):
			if fnmatch.fnmatch(name, pat):
				if Exists(fullname) or (return_folders and Path.isdir(fullname)):
					result.append(fullname)
				continue
		# recursively scan other folders, appending results
		if recurse:
			if Path.isdir(fullname) and not Path.islink(fullname):
				result = result + Walk( fullname, recurse, pattern, return_folders )
	return result

if __name__ == '__main__':
	pass